@@ -35,7 +35,6 @@
 #' CIbeta(m)
 #'
 #' @export
-#' @importFrom MASS ginv
 #' @importFrom utils tail
 
 CIbeta <- function(m,alpha=0.95)
@@ -69,7 +68,7 @@ CIbeta <- function(m,alpha=0.95)
 
 
   # inverse of Hessian
-  if(!is.null(m$mod$hessian)) Sigma <- ginv(m$mod$hessian)
+  if(!is.null(m$mod$hessian)) Sigma <- m$mod$Sigma
   else Sigma <- NULL
 
   p <- parDef(dist,nbStates,m$conditions$estAngleMean,m$conditions$zeroInflation,m$conditions$oneInflation,m$conditions$DM,m$conditions$userBounds)
@@ -126,6 +125,9 @@ CIbeta <- function(m,alpha=0.95)
     est <- w2wn(wpar[tail(cumsum(unlist(parCount)),1)+1:((nbCovs+1)*nbStates*(nbStates-1))],m$conditions$workBounds$beta)
     
     Par$beta <- get_CIwb(wpar[tail(cumsum(unlist(parCount)),1)+1:((nbCovs+1)*nbStates*(nbStates-1))],est,tail(cumsum(unlist(parCount)),1)+1:((nbCovs+1)*nbStates*(nbStates-1)),Sigma,alpha,m$conditions$workBounds$beta,rnames=rownames(m$mle$beta),cnames=colnames(m$mle$beta),cons=rep(1,length(est)))
+    
+    # fill in constraints based on betaCons
+    Par$beta <- lapply(Par$beta,function(x) matrix(x[c(m$conditions$betaCons)],dim(x),dimnames=list(rownames(x),colnames(x))))
   }
   
   # group CIs for initial distribution
