@@ -20,6 +20,8 @@
 #' Instead of a list object, \code{retrySD} can also be a scalar or a vector, in which case the same values are used for each each individual.
 #' If a scalar is provided, then the same value is used for each parameter. If a vector is provided, it must be of length \code{length(theta)} for the corresponding individual(s). Default: 1, i.e., a standard deviation of 1 is used
 #' for all parameters of all individuals. Ignored unless \code{retryFits>0}.
+#' @param retryParallel Logical indicating whether or not to perform \code{retryFits} attempts for each individual in parallel. Default: FALSE. Ignored unless \code{retryFits>0}.
+#' Note that when attempts are done in parallel (i.e. \code{retryParallel=TRUE}), the current value for the log-likelihood of each individual and warnings about convergence are not printed to the console.
 #' @param mov.model List of mov.model objects (see \code{\link[crawl]{crwMLE}}) containing an element for each individual. If only one movement model is provided, then the same movement model is used
 #' for each individual.
 #' @param err.model List of err.model objects (see \code{\link[crawl]{crwMLE}}) containing an element for each individual. If only one error model is provided, then the same error model is used
@@ -31,8 +33,7 @@
 #' @param coord A 2-vector of character values giving the names of the "x" and
 #' "y" coordinates in \code{data}. See \code{\link[crawl]{crwMLE}}.
 #' @param Time.name Character indicating name of the location time column.  See \code{\link[crawl]{crwMLE}}.
-#' @param initial.state List of initial.state objects (see \code{\link[crawl]{crwMLE}}) containing an element for each individual. If only one initial state is provided, then the same initial states are used
-#' for each individual.
+#' @param time.scale character. Scale for conversion of POSIX time to numeric for modeling. Defaults to "hours".
 #' @param theta List of theta objects (see \code{\link[crawl]{crwMLE}}) containing an element for each individual. If only one theta is provided, then the same starting values are used
 #' for each individual. If theta is not specified, then \code{\link[crawl]{crwMLE}} default values are used (i.e. each parameter is started at zero).
 #' @param fixPar List of fixPar objects (see \code{\link[crawl]{crwMLE}}) containing an element for each individual. If only one fixPar is provided, then the same parameters are held fixed to the given value
@@ -55,6 +56,7 @@
 #' @param fillCols Logical indicating whether or not to use the crawl::\code{\link[crawl]{fillCols}} function for filling in missing values in \code{obsData} for which
 #' there is a single unique value. Default: FALSE. If the output from \code{crawlWrap} is intended for analyses using \code{\link{fitHMM}} or \code{\link{MIfitHMM}}, 
 #' setting \code{fillCols=TRUE} should typically be avoided.
+#' @param ... Additional arguments that are ignored.
 #' 
 #' @return A \code{\link{crwData}} object, i.e. a list of:
 #' \item{crwFits}{A list of \code{crwFit} objects returned by crawl::crwMLE. See \code{\link[crawl]{crwMLE}}}
@@ -65,7 +67,7 @@
 #' \itemize{
 #' \item Consult \code{\link[crawl]{crwMLE}} and \code{\link[crawl]{crwPredict}} for futher details about model fitting and prediction. 
 #' 
-#' \item Note that the names of the list elements corresponding to each individual in \code{mov.model}, \code{err.model}, \code{activity}, \code{drift}, \code{initial.state},
+#' \item Note that the names of the list elements corresponding to each individual in \code{mov.model}, \code{err.model}, \code{activity}, \code{drift},
 #' \code{theta}, \code{fixPar}, \code{constr}, \code{prior}, and \code{predTime} must match the individual IDs in \code{obsData}.  If only one element is provided
 #' for any of these arguments, then the same element will be applied to all individuals.
 #' }
@@ -77,22 +79,19 @@
 #' # extract simulated obsData from example data
 #' obsData <- miExample$obsData
 #' 
-#' # extract crwMLE inputs from example data
-#' inits <- miExample$inits # initial state
-#' err.model <- miExample$err.model # error ellipse model
+#' # error ellipse model
+#' err.model <- list(x= ~ ln.sd.x - 1, y =  ~ ln.sd.y - 1, rho =  ~ error.corr)
 #'
 #' # Fit crwMLE models to obsData and predict locations 
 #' # at default intervals for both individuals
 #' crwOut1 <- crawlWrap(obsData=obsData,
 #'          theta=c(4,0),fixPar=c(1,1,NA,NA),
-#'          initial.state=inits,
 #'          err.model=err.model,attempts=100)
 #'
 #' # Fit the same crwMLE models and predict locations 
 #' # at same intervals but specify for each individual using lists
 #' crwOut2 <- crawlWrap(obsData=obsData,
 #'          theta=list(c(4,0),c(4,0)), fixPar=list(c(1,1,NA,NA),c(1,1,NA,NA)),
-#'          initial.state=list(inits,inits),
 #'          err.model=list(err.model,err.model),
 #'          predTime=list('1'=seq(1,633),'2'=seq(1,686)))
 #' }
@@ -101,16 +100,17 @@
 #' @importFrom crawl crwMLE crwPredict displayPar
 #' @importFrom doParallel registerDoParallel stopImplicitCluster
 #' @importFrom foreach foreach %dopar%
-#' @importFrom stats formula
+#' @importFrom doRNG %dorng%
+#' @importFrom stats formula setNames
 #' @importFrom sp coordinates
 #' @importFrom utils capture.output
 
-crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1,
+crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1, retryParallel = FALSE,
                     mov.model = ~1, err.model = NULL, activity = NULL, drift = NULL, 
-                    coord = c("x", "y"), Time.name = "time", initial.state, theta, fixPar, 
+                    coord = c("x", "y"), Time.name = "time", time.scale = "hours", theta, fixPar, 
                     method = "L-BFGS-B", control = NULL, constr = NULL, 
                     prior = NULL, need.hess = TRUE, initialSANN = list(maxit = 200), attempts = 1,
-                    predTime = NULL, fillCols = FALSE)
+                    predTime = NULL, fillCols = FALSE, ...)
 {
   
   if(is.data.frame(obsData)){
@@ -197,20 +197,6 @@ crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1,
     drift <- drift[ids]
   } else names(drift) <- ids
   
-  if(!is.null(names(initial.state))){
-    if(all(names(initial.state) %in% c("a","P"))){
-      tmpinitial.state<-initial.state
-      initial.state<-list()
-      for(i in ids){
-        initial.state[[i]] <- tmpinitial.state
-      } 
-    }
-  }
-  if(!is.null(names(initial.state)))  {
-    if(!all(names(initial.state) %in% ids)) stop("initial.state names must match obsData$ID")
-    initial.state <- initial.state[ids]
-  } else names(initial.state) <- ids
-  
   if(!missing(fixPar)){
     if(!is.list(fixPar)){
       tmpfixPar<-fixPar
@@ -354,9 +340,9 @@ crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1,
   
   cat('Fitting',length(ids),'track(s) using crawl::crwMLE...',ifelse(ncores>1,"","\n"))
   registerDoParallel(cores=ncores) 
-  model_fits <- 
-    foreach(i = ids) %dopar% {
-      
+  withCallingHandlers(model_fits <- 
+    foreach(i = ids, .export="crwMLE", .errorhandling="pass", .final = function(x) stats::setNames(x, ids)) %dorng% {
+      cat("Individual ",i,"...\n",sep="")
       fit <- crawl::crwMLE(
         mov.model =  mov.model[[i]],
         err.model = err.model[[i]],
@@ -365,7 +351,7 @@ crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1,
         data = ind_data[[i]],
         coord = coord,
         Time.name = Time.name,
-        initial.state = initial.state[[i]],
+        time.scale = time.scale,
         theta = theta[[i]],
         fixPar = fixPar[[i]],
         method = method,
@@ -374,29 +360,38 @@ crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1,
         prior = prior[[i]],
         need.hess = need.hess,
         initialSANN = initialSANN,
-        attempts = attempts
+        attempts = attempts, 
+        ... = ...
       )
-      
     }
+  ,warning=muffleRNGwarning)
   stopImplicitCluster()
   cat("DONE\n")
+  if(retryFits>0) cat("\n\n")
   
-  names(model_fits) <- ids
+  if(retryParallel & ncores>1){
+    for(i in ids){
+      if(!inherits(model_fits[[i]],"crwFit"))
+        warning('crawl::crwMLE for individual ',i,' failed;\n',model_fits[[i]],"   Check crawl::crwMLE arguments and/or consult crawl documentation.")
+    }
+  }
   
   # Check crwFits and re-try based on retryFits
-  for(i in ids){
-    if(!inherits(model_fits[[i]],"crwFit"))
-      warning('crawl::crwMLE for individual ',i,' failed;\n',model_fits[[i]],"   Check crawl::crwMLE arguments and/or consult crawl documentation.")
-    else {
+  tmpcores <- ncores
+  if(!retryParallel) tmpcores <- 1
+  else if(ncores>1 & length(ids)>1) cat("Attempting to achieve convergence and valid variance estimates for each individual in parallel.\n    Press 'esc' to force exit from 'crawlWrap'... ",sep="")
+  registerDoParallel(cores=tmpcores) 
+  withCallingHandlers(model_fits <- foreach(i = ids, .export=c("quietCrawl","crwMLE"), .errorhandling="pass", .final = function(x) stats::setNames(x, ids)) %dorng% {
+    if(inherits(model_fits[[i]],"crwFit")){
       if((model_fits[[i]]$convergence | any(is.na(model_fits[[i]]$se[which(is.na(fixPar[[i]]))]))) | retryFits){
         if(retryFits){
           fitCount<-0
           fit <- model_fits[[i]]
           if(model_fits[[i]]$convergence | any(is.na(model_fits[[i]]$se[which(is.na(fixPar[[i]]))]))){
             if(model_fits[[i]]$convergence)
-              cat('crawl::crwMLE for individual',i,'has suspect convergence: ',model_fits[[i]]$message,"\n")
+              cat('\ncrawl::crwMLE for individual',i,'has suspect convergence: ',model_fits[[i]]$message,"\n")
             if(any(is.na(model_fits[[i]]$se[which(is.na(fixPar[[i]]))])))
-              cat('crawl::crwMLE for individual',i,'has NaN variance estimate(s)\n')
+              cat('\ncrawl::crwMLE for individual',i,'has NaN variance estimate(s)\n')
             cat('Attempting to achieve convergence and valid variance estimates for individual ',i,". Press 'esc' to force exit from 'crawlWrap'\n",sep="")
           } else {
             cat('Attempting to improve fit for individual ',i,". Press 'esc' to force exit from 'crawlWrap'\n",sep="")
@@ -404,37 +399,45 @@ crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1,
           curFit<-fit
           while(fitCount<retryFits){ # & (fit$convergence | any(is.na(fit$se[which(is.na(fixPar[[i]]))])))){
             cat("\r    Attempt ",fitCount+1," of ",retryFits," -- current log-likelihood value: ",curFit$loglik,"  ...",sep="")
-            tmp <- tryCatch(suppressWarnings(suppressMessages(crawl::crwMLE(
-              mov.model =  mov.model[[i]],
-              err.model = err.model[[i]],
-              activity = activity[[i]],
-              drift = drift[[i]],
-              data = ind_data[[i]],
-              coord = coord,
-              Time.name = Time.name,
-              initial.state = initial.state[[i]],
-              theta = fit$estPar + rnorm(length(fit$estPar),0,retrySD[[i]]),
-              fixPar = fixPar[[i]],
-              method = method,
-              control = control,
-              constr = constr[[i]],
-              prior = prior[[i]],
-              need.hess = need.hess,
-              initialSANN = initialSANN,
-              attempts = 1
-            ))),error=function(e) e)
+            tmpFun <- function(){
+              tryCatch(suppressWarnings(suppressMessages(
+                crawl::crwMLE(mov.model =  mov.model[[i]],
+                              err.model = err.model[[i]],
+                              activity = activity[[i]],
+                              drift = drift[[i]],
+                              data = ind_data[[i]],
+                              coord = coord,
+                              Time.name = Time.name,
+                              time.scale = time.scale,
+                              theta = fit$estPar + rnorm(length(fit$estPar),0,retrySD[[i]]),
+                              fixPar = fixPar[[i]],
+                              method = method,
+                              control = control,
+                              constr = constr[[i]],
+                              prior = prior[[i]],
+                              need.hess = need.hess,
+                              initialSANN = initialSANN,
+                              attempts = 1, 
+                              ... = ...))),error=function(e){e})}
+            tmp <- NULL
+            if(retryParallel){
+              tmp <- tmpFun()
+            } else {
+              # hack to suppress crawl cat output
+              tmp <- quietCrawl(tmpFun())
+            }
             if(inherits(tmp,"crwFit")){
               if(tmp$convergence==0){
-                if(tmp$aic < model_fits[[i]]$aic | all(!is.na(tmp$se[which(is.na(fixPar[[i]]))])))
+                if(tmp$loglik > curFit$loglik | all(!is.na(tmp$se[which(is.na(fixPar[[i]]))])))
                   fit<-tmp
-                if(tmp$aic <= model_fits[[i]]$aic & all(!is.na(tmp$se[which(is.na(fixPar[[i]]))])))
+                if(tmp$loglik >= curFit$loglik & all(!is.na(tmp$se[which(is.na(fixPar[[i]]))])))
                   curFit<-tmp
               }
             }
             fitCount <- fitCount + 1
           }
           if(curFit$convergence | any(is.na(curFit$se[which(is.na(fixPar[[i]]))]))){
-            cat("FAILED\n")
+            message("FAILED\n")
           } else {
             cat("DONE\n")
           }
@@ -446,9 +449,14 @@ crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1,
             warning('crawl::crwMLE for individual ',i,' has NaN variance estimate(s)')
         }
       }
+    } else {
+      warning('\ncrawl::crwMLE for individual ',i,' failed;\n',model_fits[[i]],"   Check crawl::crwMLE arguments and/or consult crawl documentation.\n")
     }
-  }
-  
+    model_fits[[i]]
+  },warning=muffleRNGwarning)
+  stopImplicitCluster()
+  if(retryParallel & ncores>1 & length(ids)>1) cat("DONE\n")
+
   convFits <- ids[which(unlist(lapply(model_fits,function(x) inherits(x,"crwFit"))))]
   model_fits <- model_fits[convFits]
   
@@ -464,16 +472,27 @@ crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1,
     }
     if(length(unique(td))==1) txt <- paste('at',td[[1]],'time steps')
   }
-  cat('Predicting locations (and uncertainty)',txt,'for',length(convFits),'track(s) using crawl::crwPredict...')
+  if(retryFits>0) cat("\n")
+  cat('\nPredicting locations (and uncertainty)',txt,'for',length(convFits),'track(s) using crawl::crwPredict... ')
+  
+  if (time.scale %in% c("hours", "hour")) {
+    ts <- 60 * 60
+  } else if (time.scale %in% c("days", "day")) {
+    ts <- 60 * 60 * 24
+  } else if (time.scale %in% c("sec", "secs", "second","seconds")) {
+    ts <- 1
+  } else if (time.scale %in% c("min", "mins", "minute", "minutes")) {
+    ts <- 60
+  }
   
   registerDoParallel(cores=ncores)
-  predData <- 
-    foreach(i = convFits, .export="crwPredict", .combine = rbind, .errorhandling="remove") %dopar% {
-      pD<-crawl::crwPredict(model_fits[[i]], predTime=predTime[[i]])
+  withCallingHandlers(predData <- 
+    foreach(i = convFits, .export="crwPredict", .combine = rbind, .errorhandling="remove") %dorng% {
+      pD<-crawl::crwPredict(model_fits[[i]], predTime=predTime[[i]],return.type = "flat")
       if(length(predTime[[i]])>1){
         pD[[Time.name]][which(pD$locType=="p")]<-predTime[[i]][predTime[[i]]>=min(model_fits[[i]]$data[[Time.name]])]
       } else if(inherits(obsData[[Time.name]],"POSIXct")){
-        pD[[Time.name]] <- as.POSIXct(pD$TimeNum,origin="1970-01-01 00:00:00",tz=attributes(pD[[Time.name]])$tzone)
+        pD[[Time.name]] <- as.POSIXct(pD$TimeNum*ts,origin="1970-01-01 00:00:00",tz=attributes(pD[[Time.name]])$tzone)
       }
       if(!fillCols){
         for(j in names(pD)[names(pD) %in% names(ind_data[[i]])]){
@@ -487,8 +506,15 @@ crawlWrap<-function(obsData, timeStep=1, ncores = 1, retryFits = 0, retrySD = 1,
       }
       pD
     }
+  ,warning=muffleRNGwarning)
   stopImplicitCluster()
   cat("DONE\n")
   
   return(crwData(list(crwFits=model_fits,crwPredict=predData)))
-}
\ No newline at end of file
+}
+
+quietCrawl <- function(x) { 
+  sink(tempfile()) 
+  on.exit(sink()) 
+  invisible(force(x))
+} 
\ No newline at end of file
