@@ -56,14 +56,11 @@ exGen <- function()
 
   example <- list(m=m,simPar=simPar,par0=par0)
   
-  inits <- list(a=c(0,0,0,0),P = diag(c(5000 ^ 2,10 * 3600 ^ 2, 5000 ^ 2, 10 * 3600 ^ 2)))
   err.model <- list(x= ~ ln.sd.x - 1, y =  ~ ln.sd.y - 1, rho =  ~ error.corr)
   
   obsData<-simObsData(data,lambda=2,errorEllipse=list(M=c(0,50),m=c(0,50),r=0))
   
-  crwOut <- crawlWrap(obsData,theta=c(4,0),fixPar=c(1,1,NA,NA),
-   initial.state=inits,
-   err.model=err.model)
+  crwOut <- crawlWrap(obsData,theta=c(4,0),fixPar=c(1,1,NA,NA),err.model=err.model)
   
   bestData<-prepData(crwOut,covNames=c("cov1","cov2"))
   bestFit<-fitHMM(bestData,nbStates=nbStates,Par0=list(step=stepPar0,angle=anglePar0),beta0=beta0,
@@ -72,10 +69,10 @@ exGen <- function()
   bPar<-getPar(bestFit)
   
   miFits<-MIfitHMM(crwOut,nSims=4,nbStates=nbStates,Par0=bPar$Par,beta0=bPar$beta,
-                    delta0=bPar$delta,formula=formula,dist=list(step=stepDist,angle=angleDist),estAngleMean=list(angle=TRUE),
+                    formula=formula,dist=list(step=stepDist,angle=angleDist),estAngleMean=list(angle=TRUE),
                     covNames=c("cov1","cov2"))
   
-  miExample <- list(obsData=obsData,bPar=bPar,inits=inits,err.model=err.model)
+  miExample <- list(obsData=obsData,bPar=bPar)
   
   set.seed(3)
   buffer<-100000        # grid half-width
@@ -114,8 +111,7 @@ exGen <- function()
 #' \code{miExample} is a list of the following objects for demonstrating \code{\link{crawlWrap}}, \code{\link{MIfitHMM}}, and \code{\link{MIpool}}:
 #' \itemize{
 #' \item \code{obsData} Simulated observation data with measurement error and temporal irregularity (generated by \code{\link{simData}})
-#' \item \code{inits} initial states for \code{crawlWrap} example
-#' \item \code{err.model} Error ellipse model for \code{crawlWrap} example
+#' \item \code{bPar} initial parameter estimates for \code{MIfitHMM} examples
 #' }
 #' 
 #' \code{forest} is a simulated spatial covariate raster layer
