@@ -29,25 +29,17 @@ print.momentuHMM.version <- function()
   print.momentuHMM.version()
 }
 
-# this function maintains backwards compatibility with momentuHMM versions <1.1.2 (formulaDelta) and <1.4.0 (workBounds)
+# suppress RNG warning when using %dorng%
+muffleRNGwarning <- function(w) {
+  if(any(grepl("Foreach loop had changed the current RNG type: RNG was restored to same type, next state",w)))
+    invokeRestart("muffleWarning")
+}
+
+#' @importFrom MASS ginv
+# this function maintains backwards compatibility with momentuHMM versions <1.4.0 (workBounds) and <1.4.3 (betaCons)
 delta_bc <- function(m){
   
   if(is.momentuHMM(m) | is.miSum(m)){
-    if(is.null(m$conditions$formulaDelta)) {
-      m$conditions$formulaDelta <- ~1
-      aInd <- NULL
-      nbAnimals <- length(unique(m$data$ID))
-      for(i in 1:nbAnimals)
-        aInd <- c(aInd,which(m$data$ID==unique(m$data$ID)[i])[1])
-      m$covsDelta <- model.matrix(m$conditions$formulaDelta,m$data[aInd,,drop=FALSE]) 
-      if(is.miSum(m)){
-        m$Par$real$delta$est <- matrix(m$Par$real$delta$est,nrow=nrow(m$covsDelta),ncol=length(m$stateNames),byrow=TRUE,dimnames = list(paste0("ID:",unique(m$data$ID)),m$stateNames))
-      } else {
-        m$mle$delta <- matrix(m$mle$delta,nrow=nrow(m$covsDelta),ncol=length(m$stateNames),byrow=TRUE,dimnames = list(paste0("ID:",unique(m$data$ID)),m$stateNames))
-        #m$CIreal <- CIreal(m)
-        m$CIbeta <- CIbeta(m)
-      }
-    }
     if(is.null(m$conditions$workBounds)){
       distnames <- names(m$conditions$dist)
       
@@ -69,7 +61,25 @@ delta_bc <- function(m){
       }
       m$conditions$workBounds <- getWorkBounds(workBounds,distnames,m$mod$estimate,parindex,parCount,m$conditions$DM,beta,delta)
     }
-
+    if(is.null(m$conditions$betaCons)){
+      if(is.miSum(m) & !is.null(m$Par$beta$beta)) m$conditions$betaCons <- matrix(1:length(m$Par$beta$beta$est),nrow(m$Par$beta$beta$est),ncol(m$Par$beta$beta$est))
+      else if(is.momentuHMM(m) & !is.null(m$mle$beta)) m$conditions$betaCons <- matrix(1:length(m$mle$beta),nrow(m$mle$beta),ncol(m$mle$beta))
+    }
+    if(is.null(m$conditions$betaRef)) m$conditions$betaRef <- as.integer(1:length(m$stateNames))
+    if(is.momentuHMM(m)){
+      if(is.null(m$mod$wpar)) m$mod$wpar <- m$mod$estimate
+      if(is.null(m$mod$Sigma) & !is.null(m$mod$hessian)) m$mod$Sigma <- MASS::ginv(m$mod$hessian)
+    } else {
+      ####### compatability hack for change to MIcombine in momentuHMM >= 1.4.3 ######
+      if(is.null(m$conditions$optInd)){
+        for(i in names(m$conditions$dist)){
+          m$conditions$cons[[i]]<-rep(1,length(m$conditions$cons[[i]]))
+          m$conditions$workcons[[i]]<-rep(0,length(m$conditions$workcons[[i]]))
+          m$conditions$workBounds[[i]]<-matrix(c(-Inf,Inf),nrow(m$conditions$workBounds[[i]]),2,byrow=TRUE)
+        }
+      }
+      ################################################################################
+    }
   } else if(!is.miHMM(m) & any(unlist(lapply(m,is.momentuHMM)))){
     m <- HMMfits(m)
   }
