@@ -1,7 +1,7 @@
 
 #' Negative log-likelihood function
 #'
-#' @param wpar Vector of working parameters.
+#' @param optPar Vector of working parameters.
 #' @param nbStates Number of states of the HMM.
 #' @param formula Regression formula for the transition probability covariates. 
 #' @param bounds Named list of 2-column matrices specifying bounds on the natural (i.e, real) scale of the probability 
@@ -36,6 +36,9 @@
 #' @param covsDelta data frame containing the delta model covariates (if any)
 #' @param workBounds named list of 2-column matrices specifying bounds on the working scale of the probability distribution, transition probability, and initial distribution parameters
 #' @param prior A function that returns the log-density of the working scale parameter prior distribution(s)
+#' @param betaCons Matrix of the same dimension as \code{beta0} composed of integers identifying any equality constraints among the t.p.m. parameters.
+#' @param betaRef Indices of reference elements for t.p.m. multinomial logit link.
+#' @param optInd indices of constrained parameters
 #'
 #' @return The negative log-likelihood of the parameters given the data.
 #'
@@ -62,13 +65,14 @@
 #'      m$conditions$zeroInflation,m$conditions$oneInflation,m$conditions$stationary,
 #'      m$conditions$cons,m$conditions$fullDM,m$conditions$DMind,m$conditions$workcons,
 #'      m$conditions$Bndind,m$knownStates,unlist(m$conditions$fixPar),
-#'      m$conditions$wparIndex,covsDelta=m$covsDelta,workBounds=m$conditions$workBounds)
+#'      m$conditions$wparIndex,covsDelta=m$covsDelta,workBounds=m$conditions$workBounds,
+#'      betaRef=m$conditions$betaRef)
 #' }
 #'
 
-nLogLike <- function(wpar,nbStates,formula,bounds,parSize,data,dist,covs,
+nLogLike <- function(optPar,nbStates,formula,bounds,parSize,data,dist,covs,
                      estAngleMean,circularAngleMean,consensus,zeroInflation,oneInflation,
-                     stationary=FALSE,cons,fullDM,DMind,workcons,Bndind,knownStates,fixPar,wparIndex,nc,meanind,covsDelta,workBounds,prior=NULL)
+                     stationary=FALSE,cons,fullDM,DMind,workcons,Bndind,knownStates,fixPar,wparIndex,nc,meanind,covsDelta,workBounds,prior=NULL,betaCons=NULL,betaRef,optInd=NULL)
 {
   
   # check arguments
@@ -78,7 +82,7 @@ nLogLike <- function(wpar,nbStates,formula,bounds,parSize,data,dist,covs,
   nbCovs <- ncol(covs)-1
   
   # convert the parameters back to their natural scale
-  if(length(wparIndex)) wpar[wparIndex] <- fixPar[wparIndex]
+  wpar <- expandPar(optPar,optInd,fixPar,wparIndex,betaCons,nbStates,covsDelta,stationary,nbCovs)
   par <- w2n(wpar,bounds,parSize,nbStates,nbCovs,estAngleMean,circularAngleMean,consensus,stationary,cons,fullDM,DMind,workcons,nrow(data),dist,Bndind,nc,meanind,covsDelta,workBounds)
 
   nbAnimals <- length(unique(data$ID))
@@ -106,7 +110,7 @@ nLogLike <- function(wpar,nbStates,formula,bounds,parSize,data,dist,covs,
 
   nllk <- nLogLike_rcpp(nbStates,as.matrix(covs),data,names(dist),dist,
                         par,
-                        aInd,zeroInflation,oneInflation,stationary,knownStates)
+                        aInd,zeroInflation,oneInflation,stationary,knownStates,betaRef)
 
   if(!is.null(prior)) nllk <- nllk - prior(wpar)
   
