@@ -7,16 +7,17 @@ cosinorSin<-function(x,period){
   sin(2*pi*x/period)
 }
 
-#angleStrength<-function(angle,strength){
-#  tt <- terms(f,specials=c("angleStrength"))
-#  st <- strip.terms(tt,specials=c("angleStrength"),arguments=list(angleStrength=list("strength"=1)))
+#angleFormula<-function(angle,strength){
+#  tt <- terms(f,specials=c("angleFormula"))
+#  st <- strip.terms(tt,specials=c("angleFormula"),arguments=list(angleFormula=list("strength"=1)))
 #}
 
 stateFormulas<-function(formula,nbStates,spec="state",angleMean=FALSE,data=NULL){
   
-  Terms <- terms(formula, specials = c(paste0(spec,1:nbStates),"cosinor","angleStrength"))
+  Terms <- terms(formula, specials = c(paste0(spec,1:nbStates),"cosinor","angleFormula"))
+  if(any(grepl("angleStrength\\(",attr(Terms,"term.labels")))) stop("'angleStrength' is defunct in momentuHMM >=1.4.2. Please use 'angleFormula' instead")
   if(any(attr(Terms,"order")>1)){
-    if(any(grepl("angleStrength\\(",attr(Terms,"term.labels")[attr(Terms,"order")>1]))) stop("interactions with angleFormula are not allowed")
+    if(any(grepl("angleFormula\\(",attr(Terms,"term.labels")[attr(Terms,"order")>1]))) stop("interactions with angleFormula are not allowed")
   }
   
   stateFormula<-list()
@@ -25,8 +26,8 @@ stateFormulas<-function(formula,nbStates,spec="state",angleMean=FALSE,data=NULL)
     mainpart <- varnames
     cosInd <- survival::untangle.specials(Terms,"cosinor",order=1:10)$terms
     if(length(cosInd) & angleMean) stop("cosinor models are not supported for angle means")
-    angInd <- survival::untangle.specials(Terms,"angleStrength",order=1:10)$terms
-    if(length(angInd) & !angleMean) stop("angleStrength models are only allowed for angle means")
+    angInd <- survival::untangle.specials(Terms,"angleFormula",order=1:10)$terms
+    if(length(angInd) & !angleMean) stop("angleFormula models are only allowed for angle means")
     stateInd <- numeric()
     for(j in 1:nbStates){
       tmpInd <- survival::untangle.specials(Terms,paste0(spec,j),order=1)$terms
@@ -46,14 +47,20 @@ stateFormulas<-function(formula,nbStates,spec="state",angleMean=FALSE,data=NULL)
     for(j in varnames[cosInd])
       mainpart<-c(mainpart,paste0(gsub("cosinor","cosinorCos",j)),paste0(gsub("cosinor","cosinorSin",j)))
     if(length(angInd)){
-      stmp <- prodlim::strip.terms(Terms[attr(Terms,"specials")$angleStrength],specials="angleStrength",arguments=list(angleStrength=list("strength"=NULL,"by"=NULL)))
+      stmp <- prodlim::strip.terms(Terms[attr(Terms,"specials")$angleFormula],specials="angleFormula",arguments=list(angleFormula=list("strength"=NULL,"by"=NULL)))
       if(any(grepl("cos",attr(stmp,"term.labels"))) | any(grepl("sin",attr(stmp,"term.labels")))) stop("sorry, the strings 'cos' and 'sin' are reserved and cannot appear in mean angle formulas and/or covariate names")
       for(jj in attr(stmp,"term.labels")){
-        if(is.null(attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength)) stop("angleStrength is missing strength argument")
-        else if(!is.na(suppressWarnings(as.numeric((attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength))))) stop("angleStrength has invalid strength argument")
-        tmpForm <- as.formula(paste0("~-1+",attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength))
-        if(any(attr(terms(tmpForm),"order")>1)) stop("angleFormula strength argument for ",jj," cannot include term interactions; use the 'by' argument")
-        group <- attr(stmp,"stripped.arguments")$angleStrength[[jj]]$by
+        if(is.null(attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength)){
+          tmpForm <- ~ - 1
+          strengthInd <- FALSE
+        }
+        else {
+          if(!is.na(suppressWarnings(as.numeric((attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength))))) stop("angleFormula has invalid strength argument")
+          tmpForm <- as.formula(paste0("~-1+",attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength))
+          if(any(attr(terms(tmpForm),"order")>1)) stop("angleFormula strength argument for ",jj," cannot include term interactions; use the 'by' argument")
+          strengthInd <- TRUE
+        }
+        group <- attr(stmp,"stripped.arguments")$angleFormula[[jj]]$by
         if(!is.null(data)){
           DMterms <- attr(terms(tmpForm),"term.labels")
           factorterms<-names(data)[unlist(lapply(data,is.factor))]
@@ -70,8 +77,10 @@ stateFormulas<-function(formula,nbStates,spec="state",angleMean=FALSE,data=NULL)
               }
               form <- formula(paste("~ 0 + ",tmpcovj))
             }
-            if(any(model.matrix(form,data)<0)) stop(attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength," must be >=0 in order to be used in angleStrength")
-            if(any(unlist(lapply(data[all.vars(form)],function(x) inherits(x,"factor"))))) stop("angleFormula strength argument cannot be a factor; use the 'by' argument for factors")
+            if(strengthInd){
+              if(any(model.matrix(form,data)<0)) stop(attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength," must be >=0 in order to be used in angleFormula")
+              if(any(unlist(lapply(data[all.vars(form)],function(x) inherits(x,"factor"))))) stop("angleFormula strength argument cannot be a factor; use the 'by' argument for factors")
+            }
           }
           if(!is.null(group)){
             form<-formula(paste("~",group))
@@ -80,31 +89,32 @@ stateFormulas<-function(formula,nbStates,spec="state",angleMean=FALSE,data=NULL)
               factorvar<-factorcovs %in% varform
               varform<-rep(factorterms,times=unlist(lapply(data[factorterms],nlevels)))[which(factorvar)]
             }
-            if(any(!(varform %in% names(data)))) stop("angleStrength 'by' argument ",varform[which(!(varform %in% names(data)))]," not found in data")
-            if(any(!unlist(lapply(data[varform],function(x) inherits(x,"factor"))))) stop("angleStrength 'by' argument must be of class factor")
+            if(any(!(varform %in% names(data)))) stop("angleFormula 'by' argument ",varform[which(!(varform %in% names(data)))]," not found in data")
+            if(any(!unlist(lapply(data[varform],function(x) inherits(x,"factor"))))) stop("angleFormula 'by' argument must be of class factor")
           }
         }
 
         if(!is.null(group)) group <- paste0(group,":")
-        mainpart<-c(mainpart,paste0(group,attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength,":",c("sin","cos"),"(",jj,")"))
+        mainpart<-c(mainpart,paste0(group,ifelse(strengthInd,paste0(attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength,":"),""),c("sin","cos"),"(",jj,")"))
       }
     }
     
     for(j in 1:nbStates){
       tmplabs<-attr(Terms,"term.labels")[attr(Terms,"specials")[[paste0(spec,j)]]]
       if(length(tmplabs)){
-        tmp<- terms(as.formula(paste("~",substr(tmplabs,nchar(paste0(spec,j))+1,nchar(tmplabs)),collapse="+")),specials=c("cosinor","angleStrength"))
+        tmp<- terms(as.formula(paste("~",substr(tmplabs,nchar(paste0(spec,j))+1,nchar(tmplabs)),collapse="+")),specials=c("cosinor","angleFormula"))
       
         tmpnames<-attr(tmp,"term.labels")
+        if(any(grepl("angleStrength\\(",tmpnames))) stop("'angleStrength' is defunct in momentuHMM >=1.4.2. Please use 'angleFormula' instead")
         if(any(attr(tmp,"order")>1)){
-          if(any(grepl("angleStrength\\(",tmpnames[attr(tmp,"order")>1]))) stop("interactions with angleFormula are not allowed")
+          if(any(grepl("angleFormula\\(",tmpnames[attr(tmp,"order")>1]))) stop("interactions with angleFormula are not allowed")
         }
         mp<-tmpnames
         if(!is.null(unlist(attr(tmp,"specials"))) | angleMean){
           cosInd <- survival::untangle.specials(tmp,"cosinor",order=1:10)$terms
           if(length(cosInd) & angleMean) stop("cosinor models are not supported for angle means")
-          angInd <- survival::untangle.specials(tmp,"angleStrength",order=1:10)$terms
-          if(length(angInd) & !angleMean) stop("angleStrength models are only allowed for angle means")
+          angInd <- survival::untangle.specials(tmp,"angleFormula",order=1:10)$terms
+          if(length(angInd) & !angleMean) stop("angleFormula models are only allowed for angle means")
           if(length(cosInd) | length(angInd)){
             mp <- c(tmpnames[-c(cosInd,angInd)])
           }
@@ -119,25 +129,33 @@ stateFormulas<-function(formula,nbStates,spec="state",angleMean=FALSE,data=NULL)
           for(i in tmpnames[cosInd])
             mp<-c(mp,paste0(gsub("cosinor","cosinorCos",i)),paste0(gsub("cosinor","cosinorSin",i)))
           if(length(angInd)){
-            stmp <- prodlim::strip.terms(tmp[attr(tmp,"specials")$angleStrength],specials="angleStrength",arguments=list(angleStrength=list("strength"=NULL,"by"=NULL)))
+            stmp <- prodlim::strip.terms(tmp[attr(tmp,"specials")$angleFormula],specials="angleFormula",arguments=list(angleFormula=list("strength"=NULL,"by"=NULL)))
             if(any(grepl("cos",attr(stmp,"term.labels"))) | any(grepl("sin",attr(stmp,"term.labels")))) stop("sorry, the strings 'cos' and 'sin' are reserved and cannot appear in mean angle formulas and/or covariate names")
             for(jj in attr(stmp,"term.labels")){
-              if(is.null(attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength)) stop("angleStrength is missing strength argument")
-              else if(!is.na(suppressWarnings(as.numeric((attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength))))) stop("angleStrength has invalid strength argument")
-              tmpForm <- as.formula(paste0("~-1+",attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength))
-              if(any(attr(terms(tmpForm),"order")>1)) stop("angleFormula strength argument cannot include term interactions; use the 'by' argument")
-              group <- attr(stmp,"stripped.arguments")$angleStrength[[jj]]$by
+              if(is.null(attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength)){
+                tmpForm <- ~ - 1
+                strengthInd <- FALSE
+              }
+              else {
+                if(!is.na(suppressWarnings(as.numeric((attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength))))) stop("angleFormula has invalid strength argument")
+                tmpForm <- as.formula(paste0("~-1+",attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength))
+                if(any(attr(terms(tmpForm),"order")>1)) stop("angleFormula strength argument cannot include term interactions; use the 'by' argument")
+                strengthInd <- TRUE
+              }
+              group <- attr(stmp,"stripped.arguments")$angleFormula[[jj]]$by
               if(!is.null(data)){
-                if(any(model.matrix(tmpForm,data)<0)) stop(attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength," must be >=0 in order to be used in angleStrength")
-                if(inherits(data[[attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength]],"factor")) stop("angleFormula strength argument cannot be a factor; use the 'by' argument for factors")
+                if(strengthInd){
+                  if(any(model.matrix(tmpForm,data)<0)) stop(attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength," must be >=0 in order to be used in angleFormula")
+                  if(inherits(data[[attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength]],"factor")) stop("angleFormula strength argument cannot be a factor; use the 'by' argument for factors")
+                }
                 if(!is.null(group)){
                   varform <- all.vars(as.formula(paste0("~",group)))
-                  if(any(!(varform %in% names(data)))) stop("angleStrength 'by' argument ",varform[which(!(varform %in% names(data)))]," not found in data")
-                  if(any(!unlist(lapply(data[varform],function(x) inherits(x,"factor"))))) stop("angleStrength 'by' argument must be of class factor")
+                  if(any(!(varform %in% names(data)))) stop("angleFormula 'by' argument ",varform[which(!(varform %in% names(data)))]," not found in data")
+                  if(any(!unlist(lapply(data[varform],function(x) inherits(x,"factor"))))) stop("angleFormula 'by' argument must be of class factor")
                 }
               }
               if(!is.null(group)) group <- paste0(group,":")
-              mp<-c(mp,paste0(group,attr(stmp,"stripped.arguments")$angleStrength[[jj]]$strength,":",c("sin","cos"),"(",jj,")"))
+              mp<-c(mp,paste0(group,ifelse(strengthInd,paste0(attr(stmp,"stripped.arguments")$angleFormula[[jj]]$strength,":"),""),c("sin","cos"),"(",jj,")"))
             }
           }
         }
