@@ -14,12 +14,14 @@ using namespace std;
 //' @param nr number of rows in design matrix
 //' @param nc number of column in design matrix
 //' @param circularAngleMean indicator for whether or not circular-circular regression model
+//' @param consensus indicator for whether or not circular-circular regression consensus model
 //' @param rindex row index for design matrix
 //' @param cindex column index for design matrix
+//' @param nbStates number of states
 //'
 //' @return XB matrix
 // [[Rcpp::export]]
-arma::mat XBloop_rcpp(List DM, NumericVector Xvec, unsigned int nbObs, unsigned int nr, unsigned int nc, bool circularAngleMean, IntegerVector rindex, arma::mat cindex)
+arma::mat XBloop_rcpp(List DM, NumericVector Xvec, unsigned int nbObs, unsigned int nr, unsigned int nc, bool circularAngleMean, bool consensus, IntegerVector rindex, arma::mat cindex, int nbStates)
 {
   unsigned int i;
   unsigned int j;
@@ -33,34 +35,38 @@ arma::mat XBloop_rcpp(List DM, NumericVector Xvec, unsigned int nbObs, unsigned
   XB2.zeros();
   
   unsigned int nrindex = rindex.size();
+  unsigned int incr = (circularAngleMean) ? 2 : 1;
+  
+  NumericVector DMelem2;
+  unsigned int Xcount;
   
   for(i=0; i<nrindex; i++){
-    for(j=0; j<nc; j++){
+    Xcount = 0;
+    for(j=0; j<nc; j+=incr){
       if(cindex(rindex[i],j)){
         NumericVector DMelem = DM[j*nr+rindex[i]];
-        int DMsize = DMelem.size();
+        bool DMsize1 = (DMelem.size()>1);
+        bool DMsize2 = DMsize1;
+        if(circularAngleMean) {
+          DMelem2 = DM[(j+1)*nr+rindex[i]];
+          DMsize2 = (DMelem2.size()>1);
+        }
+        bool DMsize = (DMsize1 || DMsize2);
         for(k=0; k<nbObs; k++){
-          if(DMsize>1){
-            if(!circularAngleMean){
-              XB(rindex[i],k) += DMelem[k] * Xvec[j];
-            } else {
-              XB1(rindex[i],k) += sin(DMelem[k])*Xvec[j];
-              XB2(rindex[i],k) += cos(DMelem[k])*Xvec[j];
-            }
+          if(!circularAngleMean){
+            XB(rindex[i],k) += DMelem[k*DMsize] * Xvec[j];
           } else {
-            if(!circularAngleMean){
-              XB(rindex[i],k) += DMelem[0] * Xvec[j];
-            } else {
-              XB1(rindex[i],k) += sin(DMelem[0])*Xvec[j];
-              XB2(rindex[i],k) += cos(DMelem[0])*Xvec[j];
-            }
+            XB1(rindex[i],k) += DMelem[k*DMsize1]*Xvec[Xcount];
+            XB2(rindex[i],k) += DMelem2[k*DMsize2]*Xvec[Xcount];
           }
         }
       }
+      Xcount += 1;
+    }
+    if(circularAngleMean){
+      XB.row(rindex[i]) = atan2(XB1.row(rindex[i]),1.+XB2.row(rindex[i]));
+      if(consensus) XB.row(rindex[i]+nbStates) = sqrt(pow(XB1.row(rindex[i]),2.)+pow(1.+XB2.row(rindex[i]),2.));
     }
-  }
-  if(circularAngleMean){
-    XB = atan2(XB1,1.+XB2);
   }
   return XB;
 }
